package java.awt;

import sun.awt.AppContext;
import sun.awt.ModalExclude;

abstract class ModalEventFilter implements EventFilter
{
    protected Dialog modalDialog;
    protected boolean disabled;
    
    protected ModalEventFilter(Dialog modalDialog) {
        this.modalDialog = modalDialog;
        this.disabled = false;
    }
    
    Dialog getModalDialog() {
        return this.modalDialog;
    }
    
    @Override
    public EventFilter.FilterAction acceptEvent(AWTEvent awtEvent) {
        if (this.disabled || !this.modalDialog.isVisible()) {
            return EventFilter.FilterAction.ACCEPT;
        }
        int id = awtEvent.getID();
        if ((id >= 500 && id <= 507) || (id >= 1001 && id <= 1001) || id == 201) {
            Object source = awtEvent.getSource();
            if (!(source instanceof ModalExclude) && source instanceof Component) {
                Component parent_NoClientCode;
                for (parent_NoClientCode = (Component)source; parent_NoClientCode != null && !(parent_NoClientCode instanceof Window); parent_NoClientCode = parent_NoClientCode.getParent_NoClientCode()) {}
                if (parent_NoClientCode != null) {
                    return this.acceptWindow((Window)parent_NoClientCode);
                }
            }
        }
        return EventFilter.FilterAction.ACCEPT;
    }
    
    protected abstract EventFilter.FilterAction acceptWindow(Window p0);
    
    void disable() {
        this.disabled = true;
    }
    
    int compareTo(ModalEventFilter modalEventFilter) {
        Dialog dialog = modalEventFilter.getModalDialog();
        Dialog dialog2 = this.modalDialog;
        while (dialog2 != null) {
            if (dialog2 == dialog) {
                return 1;
            }
            dialog2 = dialog2.getParent_NoClientCode();
        }
        while (dialog != null) {
            if (dialog == this.modalDialog) {
                return -1;
            }
            dialog = dialog.getParent_NoClientCode();
        }
        Dialog dialog3 = this.modalDialog.getModalBlocker();
        while (dialog3 != null) {
            if (dialog3 == dialog) {
                return -1;
            }
            dialog3 = dialog3.getModalBlocker();
        }
        Dialog dialog4 = dialog.getModalBlocker();
        while (dialog4 != null) {
            if (dialog4 == this.modalDialog) {
                return 1;
            }
            dialog4 = dialog4.getModalBlocker();
        }
        return this.modalDialog.getModalityType().compareTo(dialog.getModalityType());
    }
    
    static ModalEventFilter createFilterForDialog(Dialog dialog) {
        switch (dialog.getModalityType()) {
            case DOCUMENT_MODAL: {
                return new DocumentModalEventFilter(dialog);
            }
            case APPLICATION_MODAL: {
                return new ApplicationModalEventFilter(dialog);
            }
            case TOOLKIT_MODAL: {
                return new ToolkitModalEventFilter(dialog);
            }
            default: {
                return null;
            }
        }
    }
    
    private static class ToolkitModalEventFilter extends ModalEventFilter
    {
        private AppContext appContext;
        
        ToolkitModalEventFilter(Dialog dialog) {
            super(dialog);
            this.appContext = dialog.appContext;
        }
        
        @Override
        protected EventFilter.FilterAction acceptWindow(Window owner) {
            if (owner.isModalExcluded(Dialog.ModalExclusionType.TOOLKIT_EXCLUDE)) {
                return EventFilter.FilterAction.ACCEPT;
            }
            if (owner.appContext != this.appContext) {
                return EventFilter.FilterAction.REJECT;
            }
            while (owner != null) {
                if (owner == this.modalDialog) {
                    return EventFilter.FilterAction.ACCEPT_IMMEDIATELY;
                }
                owner = owner.getOwner();
            }
            return EventFilter.FilterAction.REJECT;
        }
    }
    
    private static class ApplicationModalEventFilter extends ModalEventFilter
    {
        private AppContext appContext;
        
        ApplicationModalEventFilter(Dialog dialog) {
            super(dialog);
            this.appContext = dialog.appContext;
        }
        
        @Override
        protected EventFilter.FilterAction acceptWindow(Window owner) {
            if (owner.isModalExcluded(Dialog.ModalExclusionType.APPLICATION_EXCLUDE)) {
                return EventFilter.FilterAction.ACCEPT;
            }
            if (owner.appContext == this.appContext) {
                while (owner != null) {
                    if (owner == this.modalDialog) {
                        return EventFilter.FilterAction.ACCEPT_IMMEDIATELY;
                    }
                    owner = owner.getOwner();
                }
                return EventFilter.FilterAction.REJECT;
            }
            return EventFilter.FilterAction.ACCEPT;
        }
    }
    
    private static class DocumentModalEventFilter extends ModalEventFilter
    {
        private Window documentRoot;
        
        DocumentModalEventFilter(Dialog dialog) {
            super(dialog);
            this.documentRoot = dialog.getDocumentRoot();
        }
        
        @Override
        protected EventFilter.FilterAction acceptWindow(Window owner) {
            if (owner.isModalExcluded(Dialog.ModalExclusionType.APPLICATION_EXCLUDE)) {
                Window window = this.modalDialog.getOwner();
                while (window != null) {
                    if (window == owner) {
                        return EventFilter.FilterAction.REJECT;
                    }
                    window = window.getOwner();
                }
                return EventFilter.FilterAction.ACCEPT;
            }
            while (owner != null) {
                if (owner == this.modalDialog) {
                    return EventFilter.FilterAction.ACCEPT_IMMEDIATELY;
                }
                if (owner == this.documentRoot) {
                    return EventFilter.FilterAction.REJECT;
                }
                owner = owner.getOwner();
            }
            return EventFilter.FilterAction.ACCEPT;
        }
    }
}
